/*
 Solarbotics Firefly Twinkle Grenade Code
 
 This example code shows how to use the Firefly as a "Twinkle Twinkle Little Star" chiptune player.
 The Firefly will flash LEDs corresponding to the notes played and also will go to sleep when the button is pressed.
 The code is also a great example on how to use interrupts and sleep functions with the ATTINY45/85V at 8MHz.
 
 The circuit:
 * LED1 is attached from digital pin 3 (D4) to the 47 ohm resistot then to GND
 * LED2 is attached from digital pin 6 (D1) to the 47 ohm resistor then to GND
 * LED3 is attached from digital pin 5 (D0) to the 47 ohm resistor then to GND
 * An additional Piezo buzzer is added from pin 2 (D3) [PB3 on the expansion port] to GND 
 * The button is attached from digital pin 7 (D2) to GND
 
 Created June 2012
 By Jeremie Boulianne for Solarbotics Ltd.
 
 A majority of this sample code comes from Oomlout's ARDX CIRC-06: http://www.oomlout.com/oom.php/products/ardx/circ-06
 
 Firefly inspiration and sponsorship from Awesome Calgary winner:
 
 Calgary's Illuminated Landscapes: http://www.collaborativeprojectsyyc.blogspot.ca/
 
 Code elements & information borrowed from the following resources:
 
 High Low Tech's Programming an ATtiny with Arduino 1.0 Tutorial: http://hlt.media.mit.edu/?p=1695
 Engblaze's AVR/Arduino sleep function tutorial: http://www.engblaze.com/hush-little-microprocessor-avr-and-arduino-sleep-mode-basics/
 Engblaze's AVR/Arduino interrupt tutorial: http://www.engblaze.com/we-interrupt-this-program-to-bring-you-a-tutorial-on-arduino-interrupts/ 
 Alex Iannuzzi's reducing ATtiny power tutorial : http://www.insidegadgets.com/2011/02/05/reduce-attiny-power-consumption-by-sleeping-with-the-watchdog-timer/
 Dale Wheat's Tiny cyclon project: http://www.dalewheat.com/info/
 */
 
#include <avr/sleep.h>

volatile int btnPress = 0;                 //Button Press which must be called as a volatile variable because it is used in an Interrupt Service Routine
int speakerPin = 3;                        //Make Digital 3 the Piezo pin
int wakePin = 0;                           //Interrupt 0 which is the button on DIGITAL PIN 2!
int length = 15;                           //Number of notes
char notes[] = "ccggaagffeeddc ";          //Letters represent a note, Space represents a rest
int beats[] = { 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 4 };
int tempo = 250;

//Code needed to modify the Analog to Digital Convertor (ADC) register bits to turn it on and off
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif 

void setup() {
  pinMode(speakerPin, OUTPUT);             //Create (D3) Peizo buzzer as an output
  pinMode(4, OUTPUT);                      //Create (D4) LED1 as an output
  pinMode(1, OUTPUT);                      //Create (D1) LED2 as an output 
  pinMode(0, OUTPUT);                      //Create (D0) LED3 as an output 
  pinMode(2, INPUT);                       //Create our button pin as an input on D2
  digitalWrite(2, HIGH);                   //Enable pullup resistor for button so it isn't floating
  attachInterrupt(wakePin, wakeUp, LOW);   //Attach the interrupt to the button on D2 and execute the ISR on a LOW signal
}

void loop() {
  
  if (btnPress > 0)  {                     //Watch for button press
      delay(500);                          //Debounce the button
      sleepNow();                          //Put Firefly to sleep
      delay(500);                          //Debounce the button 
      btnPress = 0;                        //Reset button interrupt value
      } 
  for (int i = 0; i < length; i++) {
    
    if (btnPress > 0)  {                   //Watch for button press
      delay(500);                          //Debounce the button
      sleepNow();                          //Put Firefly to sleep
      delay(500);                          //Debounce the button 
      btnPress = 0;                        //Reset button interrupt value
    }
    
    if (notes[i] == ' ') {                 //If a space is found don't play a note, just delay
      digitalWrite(0, HIGH); 
      digitalWrite(1, HIGH);
      digitalWrite(4, HIGH);               //Turn ON all LEDs
      delay(beats[i] * tempo);             //Rest for a delay set by beats * tempo
    } 
    
    else {
      digitalWrite(0, HIGH); 
      digitalWrite(1, HIGH);
      digitalWrite(4, HIGH);                //Turn ON all LEDs
      playNote(notes[i], beats[i] * tempo); //Play a note at the desired tempo
    }
    digitalWrite(0, LOW); 
    digitalWrite(1, LOW);
    digitalWrite(4, LOW);                   //Turn OFF all LEDs
    delay(tempo / 2);                       //Pause between notes
  }
}

/*************************TONE SUBROUTINE***************************************/

void playTone(int tone, int duration) {      //A subroutine that plays the frequency corresponding to the tone selected
                                             //The tone is then played for a calculated duration
  for (long i = 0; i < duration * 1000L; i += tone * 2) { 
    digitalWrite(speakerPin, HIGH);
    delayMicroseconds(tone);
    digitalWrite(speakerPin, LOW);
    delayMicroseconds(tone);
  }
}

/*************************NOTE SUBROUTINE***************************************/

void playNote(char note, int duration) {     //A subroutine that creates a frequency value to correspond to the note name
  char names[] = { 'c', 'd', 'e', 'f', 'g', 'a', 'b', 'C' };
  int tones[] = { 1915, 1700, 1519, 1432, 1275, 1136, 1014, 956 }; 
  
  for (int i = 0; i < 8; i++) {              //Play one of the eight pre-set notes
    
    if (names[i] == note) {                  //Find which note to play and play it for a set duration
      playTone(tones[i], duration);
    }
  }
}


/*************************WAKE UP SUBROUTINE***************************************/

void wakeUp(){                             //ISR -> Interrupt Service Routine, called when button is pressed

  btnPress = 1;                            //Change the state of the Button Press in order to exit whatever loop is running
}

/*************************SLEEP NOW SUBROUTINE***************************************/

void sleepNow(){                            //A subroutine that puts the Arduino to sleep by turning off the CPU core and the ADC
 
  digitalWrite(0, LOW); 
  digitalWrite(1, LOW);
  digitalWrite(4, LOW);                    //Turn off all the LEDs
  
  cbi(ADCSRA,ADEN);                        //Switch ADC OFF
     
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);     //Sleep mode is set here
    
  sleep_enable();                          //Enables the sleep bit in the mcucr register so sleep is possible. just a safety pin
                            
  sleep_mode();                            //Here the device is actually put to sleep!! THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP FROM INTERRUPT
                           
  sleep_disable();                         //First thing after waking from sleep: disable sleep
                            
  sbi(ADCSRA,ADEN);                        //Switch ADC ON
  
}
